package PayServer

import (
	"encoding/json"
	"game/base/redis"
	"game/base/util"
	"net/http"
)

type MailSenderC2SMsg struct {
	Id         int64  `json:"id"`
	Attachment string `json:"attachment"`
	Title      string `json:"title"`
	Content    string `json:"content"`
	RoleIds    string `json:"roleIds"`
	Yb         int    `json:"yb"`
	All        bool   `json:"all"`
}

type MailSenderS2CMsg struct {
	Code int `json:"code"`
}

type SendWeiDuanRewardRet struct {
	Error string `json:"error"`
	Code  int    `json:"code"`
}

func (srv *PayServer) CheckSendWeiDuanReward(w http.ResponseWriter, r *http.Request) {
	ret := &SendWeiDuanRewardRet{}
	defer func() {
		ddd, _ := json.Marshal(ret)
		w.Write(ddd)
	}()

	r.ParseForm()
	sid := r.FormValue("server")
	roleId := r.FormValue("uid")
	if redis.SIsMember("weiduanReward", sid+roleId) {
		ret.Code = 11
		return
	}
	ret.Code = 1
}
func (srv *PayServer) SendWeiDuanReward(w http.ResponseWriter, r *http.Request) {
	ret := &SendWeiDuanRewardRet{}
	defer func() {
		ddd, _ := json.Marshal(ret)
		w.Write(ddd)
	}()

	r.ParseForm()
	sid := r.FormValue("server")
	roleId := r.FormValue("uid")
	name := r.FormValue("name")
	if redis.SIsMember("weiduanReward", sid+roleId) {
		ret.Code = 13
		return
	}
	id, _ := util.NextId()
	msg := &MailSenderC2SMsg{
		Id:         id,
		Title:      "微端奖励",
		Content:    "   亲爱的武极天下玩家，感谢您对我们游戏的认可和支持，在此我们将赠予您奖励，期望你在游戏中游玩愉快！",
		Attachment: "10101:100|14401:1|10201:100000",
		RoleIds:    roleId,
		All:        false,
	}

	resp, e := srv.SendMsgToGame(util.StringToInt(sid), "mail/send", msg, name)
	if e != nil {
		ret.Code = 12
		ret.Error = e.Error()
		return
	}
	rr := &MailSenderS2CMsg{}
	json.Unmarshal(resp, rr)
	ret.Code = rr.Code

	if ret.Code == 1 {
		redis.SAdd("weiduanReward", sid+roleId)
	}
	return
}
