package log

import (
	"github.com/sirupsen/logrus"
)

/*
	1、引用logrus做日志管理
	2、初始化日志包，添加调用堆栈、文件分割、elastic Hook
*/

var Log = logrus.New()  //日志logger
var Dlog = logrus.New() //数据打点logger

const (
	DataLog      = "datalog"    // 打点数据存储文件名 和elastic的index
	pipelineName = "pl_datalog" // elastic 的 ingest名
)

// 0:Panic 1:Fatal 2:Error 3:Warn 4:Info 5:Debug 6:Trace
func Init(lvl string) {
	level, err := logrus.ParseLevel(lvl)
	if err != nil {
		logrus.Error(err)
		return
	}

	Log.SetLevel(level)
	Log.SetFormatter(&logrus.TextFormatter{TimestampFormat: "2006-01-02 15:04:05"})
	if hook, err := initLineHook(); err == nil {
		Log.AddHook(hook)
	}

	// 打点logger
	Dlog.SetLevel(logrus.InfoLevel)
	if hook, err := initLineHook(); err == nil {
		Dlog.AddHook(hook)
	}
}

// 添加发送数据到切分日志的钩子
func AddLfsHook(filePath, fileName, version string) {
	if hook, err := initLfsHook(filePath, fileName, version); err == nil {
		Log.AddHook(hook)
	}
	if hook, err := initLfsHook(filePath, DataLog, version); err == nil {
		Dlog.AddHook(hook)
	}
}

// 添加发送数据到elastic的钩子
func AddElasticHook(url, host, logIndex, fileName string) {
	if hook, err := initElasticHook(url, host, logIndex, fileName); err == nil {
		Log.AddHook(hook)
	}
	if hook, err := initElasticPlHook(url, host, logIndex, DataLog); err == nil {
		Dlog.AddHook(hook)
	}
}

func NewLogEntry(fields map[string]interface{}) *logrus.Entry {
	return Log.WithFields(fields)
}

func Debug(args ...interface{}) {
	Log.Debug(args...)
}

func Debugf(format string, args ...interface{}) {
	Log.Debugf(format, args...)
}

func Info(args ...interface{}) {
	Log.Info(args...)
}

func Infof(format string, args ...interface{}) {
	Log.Infof(format, args...)
}

func Warn(args ...interface{}) {
	Log.Warn(args...)
}

func Warnf(format string, args ...interface{}) {
	Log.Warnf(format, args...)
}

func Error(args ...interface{}) {
	Log.Error(args...)
}

func Errorf(format string, args ...interface{}) {
	Log.Errorf(format, args...)
}
