package platApi

import (
	"sync"
	"time"
	"game/base/log"
)

const (
	Plat_WX = 1
	Plat_QQ = 2
)

var tokenMgr = &TokenMgr{}

func GetAppAccessToken(appID string, appSecret string, plat uint8) string {
	info, ok := tokenMgr.tokenMap.Load(appID)
	if ok {
		return info.(*TokenInfo).token
	}

	i := &TokenInfo{
		appID:     appID,
		appSecret: appSecret,
		plat:      plat,
	}
	tokenMgr.tokenMap.Store(i.appID, i)
	tokenMgr.doFreshToken()

	if info, ok := tokenMgr.tokenMap.Load(appID); ok {
		return info.(*TokenInfo).token
	}
	return ""
}
func StartAccessTokenMgr() {
	tokenMgr.tokenMap = sync.Map{}
	go func() {
		ticker := time.NewTimer(2 * time.Second)
		for {
			select {
			case <-ticker.C:
				tokenMgr.doFreshToken()
			}
		}
	}()
}

type TokenMgr struct {
	tokenMap sync.Map
}

func (t *TokenMgr) doFreshToken() {
	now := time.Now().Unix()
	t.tokenMap.Range(func(key, value interface{}) bool {
		v := value.(*TokenInfo)
		if now < v.expire {
			return true
		}
		token, expire, err := t.getAccessToke(v)
		if token != "" {
			v.token = token
			v.expire = now + int64(expire) - 1000
			log.Infof("GetWxToken %s, Expire:%d", token, expire)
		} else {
			log.Errorf("FreshToken Error app:%s,plat:%d,error:%s", v.appID, v.plat, err)
		}
		return true
	})
}

type TokenInfo struct {
	appID     string
	appSecret string
	plat      uint8
	expire    int64
	token     string
}

func (t *TokenMgr) getAccessToke(info *TokenInfo) (key string, expire uint32, err error) {
	switch info.plat {
	case Plat_WX: //微信
		key, expire, err = WXAccessToken(info.appID, info.appSecret)
	case Plat_QQ: //qq
		key, expire, err = QQAccessToken(info.appID, info.appSecret)
	case 3:
	default:
		return
	}
	return
}
