package util

import (
	"strconv"
	"time"
	"game/base/log"
)

//获取每日定点执行的cron表达式
func GetDayFixedTimeCronExpr(h, m, s int64) string {
	expr := "* * ?"
	hs := strconv.FormatInt(h, 10)
	ms := strconv.FormatInt(m, 10)
	ss := strconv.FormatInt(s, 10)

	return ss + " " + ms + " " + hs + " " + expr
}

//返回今天零点对应的时间戳
func GetTodayBeginStamp() int64 {
	t := time.Now()
	t = time.Date(t.Year(), t.Month(), t.Day(), 0, 0, 0, 0, time.Local)
	return t.Unix()
}

// 返回本周一零点对应的时间戳
func GetThisWeekBeginStamp() int64 {
	day := WhatDayOfWeek(time.Now().Unix())
	if day < 0 {
		log.Error("GetThisWeekBeginStamp day: ", day)
		return 0
	}
	nowZeroStamp := GetTodayBeginStamp()
	return nowZeroStamp - int64(day-1)*24*60*60
}

//判断一个时间戳是周几(1-7)
func WhatDayOfWeek(stamp int64) int {
	weekDays := map[string]int{
		"Monday":    1,
		"Tuesday":   2,
		"Wednesday": 3,
		"Thursday":  4,
		"Friday":    5,
		"Saturday":  6,
		"Sunday":    7,
	}

	tm := time.Unix(stamp, 0)
	day := tm.Weekday().String()

	return weekDays[day]
}

//获取本周某一日零点对应的时间戳，whatDay表示周几(1-7)
func GetWeekDayBeginStamp(whatDay uint8) int64 {
	begin := GetThisWeekBeginStamp()
	day := int64(24 * 60 * 60)
	return begin + int64(whatDay-1)*day
}

//判断一个时间戳是否在本周的时间段内
func IsInThisWeek(stamp int64) bool {
	begin := GetThisWeekBeginStamp()
	if stamp < begin {
		return false
	}

	day := int64(24 * 60 * 60)
	if stamp >= begin+7*day {
		return false
	}

	return true
}

//获取下一个整点的时间戳
func GetNextHoursStamp() int64 {
	now := time.Now()
	return now.Unix() + int64((60-now.Minute())*60-now.Second())
}

//获取下一个整点的时间差
func GetNextHoursDur() int64 {
	now := time.Now()
	return int64((60-now.Minute())*60 - now.Second())
}

func TimeStringToTime(ts string) (*time.Time, error) {
	t, e := time.ParseInLocation("2006|01|02|15|04|05", ts, time.Local)
	if e != nil {
		return nil, e
	}
	return &t, nil
}

func TimeIsDayBegin() bool {
	if time.Now().Hour() == 0 &&
		time.Now().Minute() == 0 &&
		time.Now().Second() == 0 {
		return true
	}
	return false
}
